/****************************************************************
**
** QtGantt av Gunnstein Lye
**
** This software is released under the GPL General Public License,
** see COPYING for details.
**
****************************************************************/

#include <stream.h>

#include <qapplication.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qmessagebox.h>
#include <qwidget.h>
#include <qframe.h>
#include <qpushbutton.h>
#include <qscrollbar.h>
#include <qspinbox.h>
#include <qaccel.h>
#include <qfile.h>
#include <qprinter.h>

#include "gantt.h"
#include "main.h"


MainWidget::MainWidget( char *filenamearg, QWidget *parent, const char *name )
        : QWidget( parent, name )
	{
	int menuH; // determined after menubar creation
	int menuX; //      "       "      "        "
	int ctrlH = 25; // might be better the same height as the menu?
	frameT = 5; // thickness between window edge and frame edge
	setMinimumSize( 490, 300 );

	_menubar = new QMenuBar( this );
	QRect menuRect = _menubar->frameRect();
	menuH = menuRect.height();
	menuX = menuRect.x();
	frameY = menuH + ctrlH + (2 * frameT);
	
	frame = new QFrame( this, "ganttFrame" );
	frame->setFrameStyle( QFrame::WinPanel | QFrame::Sunken );
	frame->setGeometry( frameT, frameY, 
		width()-(frameT*2), height()-frameY-frameT );
	frameW = frame->frameWidth();

	gantt = new Gantt( filenamearg, this, "gantt" );
	gantt->setGeometry( frameT+frameW, frameY+frameW,
		width()-((frameT+frameW)*2), height()-((frameY+frameW)*2) );

	f5acc = new QAccel( this );    // Draws gantt when F5 is pressed
	f5acc->connectItem( f5acc->insertItem(Key_F5), gantt, SLOT( refresh() ) );
	
	plusacc = new QAccel( this );    // Zooms in when + is pressed
	plusacc->connectItem( plusacc->insertItem(Key_Plus), this, SLOT( slotzoomIn() ) );
	minusacc = new QAccel( this );    // Zooms out when - is pressed
	minusacc->connectItem( minusacc->insertItem(Key_Minus), this, SLOT( slotzoomOut() ) );

	// CHANGE: split the gantt widget in two: a task name list and a task
	// detail widget, then separate the two with a QSplitter
	scrolltextpix = new QScrollBar( 0, 300, 1, 10, 150, QScrollBar::Horizontal, this, "scrolltextpix" );
	int stpW = 150;
	scrolltextpix->setGeometry( frameT, 5+menuH, stpW, ctrlH );
	connect( scrolltextpix, SIGNAL( valueChanged(int) ), gantt, SLOT(changetextpix(int)) );
	
	// zoom control
	spindaypix = new QSpinBox( MINDAYPIX, MAXDAYPIX, 1, this, "spindaypix" );
	spindaypix->setGeometry( frameT + stpW + 10 , 5+menuH, 50, ctrlH );
	spindaypix->setValue( 10 );
	connect( spindaypix, SIGNAL( valueChanged(int) ), gantt, SLOT(changedaypix(int)) );
	
	/* Create the menus */
	_filemenu = new QPopupMenu;
	CHECK_PTR( _filemenu );
	_filemenu->insertItem( "&New", gantt, SLOT( reset() ));
	_filemenu->insertItem( "&Open...", gantt, SLOT( open() ));
	_filemenu->insertItem( "&Print", gantt, SLOT( print() ));
	_filemenu->insertItem( "&Refresh", gantt, SLOT( refresh() ));
	_filemenu->insertItem( "&Save", gantt, SLOT( save() ));
	_filemenu->insertItem( "&Quit", qApp, SLOT( quit() ));

	_options = new QPopupMenu;
	CHECK_PTR( _options );
	weekdetailID = _options->insertItem( "&Week detail", this, SLOT( slottoggleweekdetail()) );
	isWeekdetail = FALSE;
	_options->setCheckable( TRUE ); 

	_viewmenu = new QPopupMenu;
	CHECK_PTR( _viewmenu );
	_viewmenu->insertItem( "Zoom in", this, SLOT( slotzoomIn()) );
	_viewmenu->insertItem( "Zoom out", this, SLOT( slotzoomOut()) );
	_viewmenu->insertSeparator();
	_viewmenu->insertItem( "&Options", _options );


	_helpmenu = new QPopupMenu;
	CHECK_PTR( _helpmenu );
	_helpmenu->insertItem( "&About QtGantt", this, SLOT( slotAbout() ));

	_menubar->insertItem( "&File", _filemenu );
	_menubar->insertItem( "&View", _viewmenu );
	_menubar->insertSeparator( );
	_menubar->insertItem( "&Help", _helpmenu );
	}


void MainWidget::slottoggleweekdetail()
{
	isWeekdetail = !isWeekdetail;
	_options->setItemChecked( weekdetailID, isWeekdetail );
	gantt->setweekdetail(isWeekdetail);
}

void MainWidget::slotzoomIn()
{
	int s = spindaypix->value();
	if (s < MAXDAYPIX ) {
		spindaypix->setValue( ++s );
		gantt->changedaypix(s);
	}
}

void MainWidget::slotzoomOut()
{
	int s = spindaypix->value();
	if (s > MINDAYPIX ) {
		spindaypix->setValue( --s );
		gantt->changedaypix(s);
	}
}

void MainWidget::resizeEvent( QResizeEvent * )
	{
	frame->resize( width()-(frameT*2), height()-frameY-frameT );
	gantt->resize( width()-((frameT+frameW)*2), height()-frameY-frameT-(frameW*2) );
	// Don't think we need to force a repaint, since resize will do it?
	//gantt->repaint();
	}


void MainWidget::slotAbout()
{
	QMessageBox::information( this, "About QtGantt",
				"QtGantt - a project management system\n"
				"Copyright 2000 by Gunnstein Lye, Matt Gumbley\n"
				"and the gantt-devel@geekstuff.co.uk\n"
				"mailing list\n"
				);
}


int main( int argc, char **argv )
	{
	QApplication a( argc, argv );
	int i;
	char *filename = NULL;
	
	// There may be a "proper" C++/Qt way of handling the command line, but
	// I only know the C way, so here goes...
	for (i=1; i<argc; i++)
	{
		// printf("command line arg %d is [%s]\n", i, argv[i]);
		// Again, my C++ is not as good as my C... C++ string handling?
		// Handle options
		if (argv[i][0] == '-')
		{
			switch (argv[i][1])
			{
				case 'p' : // print to this file
					if (i == (argc-1))
					{
						cerr << "Must specify an output file with -p\n";
						exit(-1);
					}
					char *output = argv[++i];
					Gantt *gantt = new Gantt( filename, 0, "gantt" );
					gantt->printToFile( output );
					exit(0);
					break;
			}
		}
		else
			filename = argv[i];
	}

	MainWidget w( a.argv()[1] );
	w.setGeometry( 0, 80, 630, 400 );
	w.setCaption( "QtGantt" );          // Sets the caption of the window
	
	a.setMainWidget( &w );
	w.show();
	return a.exec();
	}
